@@ -67,6 +67,7 @@ static Widget graph_drawfocus_choice_item;
 static Widget autoredraw_type_item;
 static Widget cursor_type_item;
 static SpinStructure *max_path_item;
+static Widget safe_mode_item;
 static Widget scrollper_item;
 static Widget shexper_item;
 static Widget linkscroll_item;
@@ -128,10 +129,12 @@ void create_props_frame(void *data)
 	force_external_viewer_item = CreateToggleButton(rc1,
             "Use external help viewer for local documents");
 #endif        
-	fr = CreateFrame(props_frame, "Limits");
+	fr = CreateFrame(props_frame, "Restrictions");
         AddDialogFormChild(props_frame, fr);
-	max_path_item = CreateSpinChoice(fr,
+        rc1 = CreateVContainer(fr);
+	max_path_item = CreateSpinChoice(rc1,
             "Max drawing path length:", 6, SPIN_TYPE_INT, 0.0, 1.0e6, 1000);
+	safe_mode_item = CreateToggleButton(rc1, "Run in safe mode");
         
 	fr = CreateFrame(props_frame, "Scroll/zoom");
         AddDialogFormChild(props_frame, fr);
@@ -201,6 +204,7 @@ void update_props_items(void)
 	SetToggleButtonState(force_external_viewer_item, force_external_viewer);
 #endif
 	SetSpinChoice(max_path_item, (double) get_max_path_limit());
+	SetToggleButtonState(safe_mode_item, safe_mode);
 	iv = (int) rint(100*scrollper);
 	SetScaleValue(scrollper_item, iv);
 	iv = (int) rint(100*shexper);
@@ -261,6 +265,7 @@ static int props_define_notify_proc(void *data)
     force_external_viewer = GetToggleButtonState(force_external_viewer_item);
 #endif
     set_max_path_limit((int) GetSpinChoice(max_path_item));
+    safe_mode = GetToggleButtonState(safe_mode_item);
     scrollper = (double) GetScaleValue(scrollper_item)/100.0;
     shexper   = (double) GetScaleValue(shexper_item)/100.0;
 
