@@ -3,7 +3,7 @@
  * 
  * Home page: http://plasma-gate.weizmann.ac.il/Grace/
  * 
- * Copyright (c) 1996-2000 Grace Development Team
+ * Copyright (c) 1996-2002 Grace Development Team
  * 
  * Maintained by Evgeny Stambulchik <fnevgeny@plasma-gate.weizmann.ac.il>
  * 
@@ -65,6 +65,7 @@ typedef struct {
     int   *pattern_defined;
     int   *pattern_empty;
     int   *pattern_full;
+    int   *colorfilter_defined;
     int    group_is_open;
     double line_width;
     Pen    pen;
@@ -92,6 +93,7 @@ static int init_svg_data(void)
         data->pattern_defined = NULL;
         data->pattern_empty = NULL;
         data->pattern_full = NULL;
+        data->colorfilter_defined = NULL;
 
         set_curdevice_data((void *) data);
     }
@@ -109,6 +111,8 @@ static int init_svg_data(void)
         data->pattern_empty[i]   = FALSE;
         data->pattern_full[i]    = FALSE;
     }
+    
+    svg_updatecmap();
 
     data->group_is_open = FALSE;
     data->line_width    = 0.0;
@@ -127,38 +131,17 @@ static int init_svg_data(void)
 /*
  * SVG conventions :
  *   Y coordinates increase downwards
- *   angles increase counterclockwise
+ *   angles increase clockwise
  */
 
 /*
- * convert coordinate system
+ * scale coordinates, using a SVG-viewer to do this gives rounding-problems
  */
-static double convertX (double x)
+static double scaleval (double val)
 {
     Svg_data *data;
     data = get_curdevice_data();
-    return x*data->side;
-}
-
-static double convertY (double y)
-{
-    Svg_data *data;
-    data = get_curdevice_data();
-    return (1.0 - y)*data->side;
-}
-
-static double convertW (double width)
-{
-    Svg_data *data;
-    data = get_curdevice_data();
-    return width*data->side;
-}
-
-static double convertH (double height)
-{
-    Svg_data *data;
-    data = get_curdevice_data();
-    return height*data->side;
+    return val*data->side;
 }
 
 int register_svg_drv(void)
@@ -166,54 +149,100 @@ int register_svg_drv(void)
     return register_device(dev_svg);
 }
 
-static void define_pattern(int i, Svg_data *data)
+void svg_updatecmap(void)
 {
-#ifndef EXPERIMENTAL_SVG_PATTERNS
-    data->pattern_full[i]  = TRUE;
-    data->pattern_defined[i] = TRUE;
-    return;
-#else
-    int j, k, l;
+    int i;
+    Svg_data *data;
+    
+    data = (Svg_data *) get_curdevice_data();
+    if (data == NULL) {
+        return;
+    } else {
+        data->colorfilter_defined = 
+            xrealloc(data->colorfilter_defined, number_of_colors()*SIZEOF_INT);
+        for (i = 0; i < number_of_colors(); i++) {
+            data->colorfilter_defined[i] = FALSE;
+        }
+    }
+}
 
-    if (data->pattern_defined[i] == TRUE) {
+static void define_pattern(int i, int c, Svg_data *data)
+{
+    int j, k, l;
+    fRGB *frgb;
+    double bg_red, bg_green, bg_blue;
+
+    if (data->pattern_defined[i] == TRUE && c < number_of_colors() && data->colorfilter_defined[c] == TRUE) {
         return;
     }
 
-    /* testing if the pattern is either empty or full */
-    data->pattern_empty[i] = TRUE;
-    data->pattern_full[i]  = TRUE;
-    for (j = 0; j < 32; j++) {
-        if (pat_bits[i][j] != 0x00) {
-            data->pattern_empty[i] = FALSE;
-        }
-        if (pat_bits[i][j] != 0xff) {
-            data->pattern_full[i] = FALSE;
+    if (data->pattern_defined[i] != TRUE) {
+        /* testing if the pattern is either empty or full */
+        data->pattern_empty[i] = TRUE;
+        data->pattern_full[i]  = TRUE;
+        for (j = 0; j < 32; j++) {
+            if (pat_bits[i][j] != 0x00) {
+                data->pattern_empty[i] = FALSE;
+            }
+            if (pat_bits[i][j] != 0xff) {
+                data->pattern_full[i] = FALSE;
+            }
         }
     }
 
     if (data->pattern_empty[i] != TRUE && data->pattern_full[i] != TRUE) {
-        /* this is an horrible hack ! */
-        /* we define pixels as squares in vector graphics */
-        fprintf(prstream,
-                "  <defs>\n   <pattern id=\"pattern%d\""
-                " width=\"%d\" height=\"%d\">\n",
-                i, 16, 16);
-        for (j = 0; j < 256; j++) {
-            k = j/16;
-            l = j%16;
-            if ((pat_bits[i][j/8] >> (j%8)) & 0x01) {
-                /* the bit is set */
-                fprintf(prstream,
-                        "     <rect x=\"%d\" y=\"%d\""
-                        " width=\"1\" height=\"1\"/>\n",
-                        l, 15 - k);
+        fprintf(prstream, "  <defs>\n");
+        /* test if the pattern is already defined. */
+        if (data->pattern_defined[i] != TRUE) {
+            /* this is an horrible hack ! */
+            /* we define pixels as squares in vector graphics */
+            /* first fill the whole pattern */
+            fprintf(prstream,
+                    "   <pattern id=\"pattern%d\" viewBox=\"0 0 16 16\""
+                    " width=\"%d\" height=\"%d\" patternUnits=\"userSpaceOnUse\">\n",
+                    i, 16, 16);
+            fprintf(prstream,"     <rect fill=\"#FFFFFF\" x=\"0\" y=\"0\""
+                            " width=\"16\" height=\"16\"/>\n");
+            for (j = 0; j < 256; j++) {
+                k = j/16;
+                l = j%16;
+                if ((pat_bits[i][j/8] >> (j%8)) & 0x01) {
+                    /* the bit is set */
+                    fprintf(prstream,
+                            "     <rect x=\"%d\" y=\"%d\""
+                            " width=\"1\" height=\"1\"/>\n",
+                            l, 15-k);
+                }
             }
+            fprintf(prstream, "   </pattern>\n");
+            data->pattern_defined[i] = TRUE;
         }
-        fprintf(prstream, "   </pattern>\n  </defs>\n");
-    }
 
-    data->pattern_defined[i] = TRUE;
-#endif
+        /* test if the needed colorfilter is already defined. */
+        /* color-patterns can be drawn with black patterns and then
+           applying a colorfilter to change white to the background-color
+           and black to the patterncolor. */
+        if (c < number_of_colors() && data->colorfilter_defined[c] != TRUE) {
+            frgb = get_frgb(getbgcolor());
+            bg_red=frgb->red;
+            bg_green=frgb->green;
+            bg_blue=frgb->blue;
+            frgb = get_frgb(c);
+            fprintf(prstream, "   <filter id=\"tocolor%d\" filterUnits=\"objectBoundingBox\"\n", c);
+            fprintf(prstream, "    color-interpolation-filters=\"sRGB\" x=\"0%%\" y=\"0%%\" width=\"100%%\" height=\"100%%\">\n");
+            fprintf(prstream, "    <feComponentTransfer>\n");
+            fprintf(prstream, "      <feFuncR type=\"discrete\" tableValues=\"%.6f %.6f\"/>\n",
+                    frgb->red, bg_red);
+            fprintf(prstream, "      <feFuncG type=\"discrete\" tableValues=\"%.6f %.6f\"/>\n",
+                    frgb->green, bg_green);
+            fprintf(prstream, "      <feFuncB type=\"discrete\" tableValues=\"%.6f %.6f\"/>\n",
+                    frgb->blue, bg_blue);
+            fprintf(prstream, "    </feComponentTransfer>\n");
+            fprintf(prstream, "   </filter>\n");
+            data->colorfilter_defined[c] = TRUE;
+        }
+        fprintf(prstream, "  </defs>\n");
+    }
 }
 
 /*
@@ -266,7 +295,7 @@ static char *escape_specials(unsigned char *s, int len)
 int svginitgraphics(void)
 {
     /* device-dependent routines */
-    devupdatecmap   = NULL;
+    devupdatecmap   = svg_updatecmap;
     
     devdrawpixel    = svg_drawpixel;
     devdrawpolyline = svg_drawpolyline;
@@ -283,14 +312,17 @@ int svginitgraphics(void)
     }
 
     fprintf(prstream, "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n");
-    fprintf(prstream, "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 20000303 Stylable//EN\"");
-    fprintf(prstream, " \"http://www.w3.org/TR/2000/03/WD-SVG-20000303/DTD/svg-20000303-stylable.dtd\">\n");
+    fprintf(prstream, "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.0//EN\"");
+    fprintf(prstream, " \"http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd\">\n");
     fprintf(prstream, "<!-- generated by %s -->\n", bi_version_string());
+    /* Let do the SVG-Viewer the conversion of coordinates. */
     fprintf(prstream, "<svg xml:space=\"preserve\" ");
     fprintf(prstream, "width=\"%.4fin\" height=\"%.4fin\" viewBox=\"%.4f %.4f %.4f %.4f\">\n",
             page_width_in, page_height_in,
             0.0, 0.0, page_width_pp, page_height_pp);
-
+    fprintf(prstream, " <g transform=\"translate(0,%.4f) scale(1,-1)\">\n",
+            page_height_pp);
+    
     /* project description */
     if (get_project_description() != NULL) {
         fprintf(prstream, " <desc>%s</desc>\n", get_project_description());
@@ -313,7 +345,7 @@ static void svg_group_props (int draw, int fill)
 
     /* do we need to redefine a group with new properties ? */
     needs_group = (data->group_is_open == TRUE) ? FALSE : TRUE;
-    lw        = data->side*getlinewidth();
+    lw        = scaleval(getlinewidth());
     fillrule  = getfillrule();
     linecap   = getlinecap();
     linejoin  = getlinejoin();
@@ -350,7 +382,7 @@ static void svg_group_props (int draw, int fill)
             data->group_is_open = FALSE;
         }
 
-        define_pattern(pen.pattern, data);
+        define_pattern(pen.pattern, pen.color, data);
         prgb = get_rgb(pen.color);
         if (prgb != NULL) {
             red   = prgb->red   >> (GRACE_BPP - 8);
@@ -367,9 +399,8 @@ static void svg_group_props (int draw, int fill)
                 fprintf(prstream, "  <g style=\"fill:#%2.2X%2.2X%2.2X",
                         red, green, blue);
             } else {
-                fprintf(prstream, "  <g style=\"color:#%2.2X%2.2X%2.2X",
-                        red, green, blue);
-                fprintf(prstream, "; fill:url(#pattern%d)", pen.pattern);
+                fprintf(prstream, "  <g style=\"filter:url(#tocolor%d); ", pen.color);
+                fprintf(prstream, "fill:url(#pattern%d)", pen.pattern);
             }
             if (getfillrule() == FILLRULE_WINDING) {
                 fprintf(prstream, "; fill-rule:nonzero");
@@ -447,8 +478,8 @@ void svg_drawpixel(VPoint vp)
     svg_group_props(FALSE, TRUE);
     fprintf(prstream,
             "   <rect x=\"%.4f\" y=\"%.4f\" width=\"%.4f\" height=\"%.4f\"/>\n",
-            convertX(vp.x), convertY (vp.y),
-            convertW(1.0), convertH(1.0));
+            scaleval(vp.x), scaleval(vp.y),
+            scaleval(1.0), scaleval(1.0));
 }
 
 void svg_drawpolyline(VPoint *vps, int n, int mode)
@@ -461,12 +492,12 @@ void svg_drawpolyline(VPoint *vps, int n, int mode)
 
     svg_group_props(TRUE, FALSE);
     fprintf(prstream, "   <path d=\"M%.4f,%.4f",
-            convertX(vps[0].x), convertY(vps[0].y));
+            scaleval(vps[0].x), scaleval(vps[0].y));
     for (i = 1; i < n; i++) {
         if (i%10 == 0) {
             fprintf(prstream, "\n            ");
         }
-        fprintf(prstream, "L%.4f,%.4f", convertX(vps[i].x), convertY(vps[i].y));
+        fprintf(prstream, "L%.4f,%.4f", scaleval(vps[i].x), scaleval(vps[i].y));
     }
     if (mode == POLYLINE_CLOSED) {
         fprintf(prstream, "z\"/>\n");
@@ -486,12 +517,12 @@ void svg_fillpolygon(VPoint *vps, int nc)
 
     svg_group_props(FALSE, TRUE);
     fprintf(prstream, "   <path  d=\"M%.4f,%.4f",
-            convertX(vps[0].x), convertY(vps[0].y));
+            scaleval(vps[0].x), scaleval(vps[0].y));
     for (i = 1; i < nc; i++) {
         if (i%10 == 0) {
             fprintf(prstream, "\n             ");
         }
-        fprintf(prstream, "L%.4f,%.4f", convertX(vps[i].x), convertY(vps[i].y));
+        fprintf(prstream, "L%.4f,%.4f", scaleval(vps[i].x), scaleval(vps[i].y));
     }
     fprintf(prstream, "z\"/>\n");
 }
@@ -515,8 +546,8 @@ void svg_drawarc(VPoint vp1, VPoint vp2, int a1, int a2)
     if ((a1 - a2)%360 == 0) {
         fprintf(prstream,
             "   <ellipse  rx=\"%.4f\" ry=\"%.4f\" cx=\"%.4f\" cy=\"%.4f\"/>\n",
-            convertW(rx), convertH(ry),
-            convertX(center.x), convertY(center.y));
+            scaleval(rx), scaleval(ry),
+            scaleval(center.x), scaleval(center.y));
     } else {
         VPoint start, end;
         
@@ -526,13 +557,13 @@ void svg_drawarc(VPoint vp1, VPoint vp2, int a1, int a2)
         end.y   = center.y + ry*sin((M_PI/180.0)*a2);
 
         fprintf(prstream,
-            "   <path  d=\"M%.4f, %.4fA%.4f, %.4f %d %d %d %.4f, %.4f\"/>\n",
-            convertX(start.x), convertY(start.y),
-            convertW(rx), convertH(ry),
+            "   <path d=\"M%.4f, %.4fA%.4f, %.4f %d %d %d %.4f, %.4f\"/>\n",
+            scaleval(start.x), scaleval(start.y),
+            scaleval(rx), scaleval(ry),
             0,
             (abs(a2 - a1) > 180) ? 1 : 0,
-            (a2 > a1) ? 0 : 1,
-            convertX(end.x), convertY(end.y));
+            (a2 > a1) ? 1 : 0,
+            scaleval(end.x), scaleval(end.y));
     }
 }
 
@@ -555,8 +586,8 @@ void svg_fillarc(VPoint vp1, VPoint vp2, int a1, int a2, int mode)
     if ((a1 - a2)%360 == 0) {
         fprintf(prstream,
             "   <ellipse  rx=\"%.4f\" ry=\"%.4f\" cx=\"%.4f\" cy=\"%.4f\"/>\n",
-            convertW(rx), convertH(ry),
-            convertX(center.x), convertY(center.y));
+            scaleval(rx), scaleval(ry),
+            scaleval(center.x), scaleval(center.y));
     } else {
         VPoint start, end;
         
@@ -567,23 +598,23 @@ void svg_fillarc(VPoint vp1, VPoint vp2, int a1, int a2, int mode)
 
         if (mode == ARCFILL_CHORD) {
             fprintf(prstream,
-                "   <path  d=\"M%.4f, %.4fA%.4f, %.4f %d %d %d %.4f, %.4fz\"/>\n",
-                convertX(start.x), convertY(start.y),
-                convertW(rx), convertH(ry),
+                "   <path d=\"M%.4f, %.4fA%.4f, %.4f %d %d %d %.4f, %.4fz\"/>\n",
+                scaleval(start.x), scaleval(start.y),
+                scaleval(rx), scaleval(ry),
                 0,
                 (abs(a2 - a1) > 180) ? 1 : 0,
-                (a2 > a1) ? 0 : 1,
-                convertX(end.x), convertY(end.y));
+                (a2 > a1) ? 1 : 0,
+                scaleval(end.x), scaleval(end.y));
         } else {
             fprintf(prstream,
-                "   <path  d=\"M%.4f,%.4fL%.4f,%.4fA%.4f,%.4f %d %d %d %.4f,%.4fz\"/>\n",
-                convertX(center.x), convertY(center.y),
-                convertX(start.x), convertY(start.y),
-                convertW(rx), convertH(ry),
+                "   <path d=\"M%.4f,%.4fL%.4f,%.4fA%.4f,%.4f %d %d %d %.4f,%.4fz\"/>\n",
+                scaleval(center.x), scaleval(center.y),
+                scaleval(start.x), scaleval(start.y),
+                scaleval(rx), scaleval(ry),
                 0,
                 (abs(a2 - a1) > 180) ? 1 : 0,
-                (a2 > a1) ? 0 : 1,
-                convertX(end.x), convertY(end.y));
+                (a2 > a1) ? 1 : 0,
+                scaleval(end.x), scaleval(end.y));
         }
     }
 }
@@ -597,16 +628,18 @@ void svg_putpixmap(VPoint vp, int width, int height, char *databits,
 void svg_puttext(VPoint vp, char *s, int len, int font,
                  TextMatrix *tm, int underline, int overline, int kerning)
 {
-    char *fontalias, *dash, *family;
-    Svg_data *data = get_curdevice_data();
-    double fsize = data->side;
+    char *fontalias, *fontfullname, *fontweight;
+    char *dash, *family, *familyff;
+    double fsize = scaleval(1);
 
     svg_group_props(FALSE, TRUE);
     
     fprintf(prstream, "   <text  ");
 
-    family  = NULL;
     fontalias = get_fontalias(font);
+    fontfullname = get_fontfullname(font);
+
+    family  = NULL;
     if ((dash = strchr(fontalias, '-')) == NULL) {
         family = copy_string(family, fontalias);
     } else {
@@ -614,25 +647,85 @@ void svg_puttext(VPoint vp, char *s, int len, int font,
         strncpy(family, fontalias, dash - fontalias);
         family[dash - fontalias] = '\0';
     }
-    fprintf(prstream, " style=\"font-family:%s", family);
+    fprintf(prstream, " style=\"font-family:'%s'", family);
+    
+    familyff=get_fontfamilyname(font);
+    if (strcmp(family,familyff) != 0){
+        fprintf(prstream, ",'%s'",familyff);
+    }
+    
     copy_string(family, NULL);
 
-    if (strstr(fontalias, "Italic") != NULL) {
-        fprintf(prstream, "; font-style:italic");
-    } else {
-        if (strstr(fontalias, "Oblique") != NULL) {
+    if (get_italic_angle(font) != 0) {
+        if ((strstr(fontfullname, "Obliqued") != NULL) ||
+            (strstr(fontfullname, "Oblique") != NULL) ||
+            (strstr(fontfullname, "Upright") != NULL) ||
+            (strstr(fontfullname, "Kursiv") != NULL) ||
+            (strstr(fontfullname, "Cursive") != NULL) ||
+            (strstr(fontfullname, "Slanted") != NULL) ||
+            (strstr(fontfullname, "Inclined") != NULL)) {
             fprintf(prstream, "; font-style:oblique");
         } else {
-            fprintf(prstream, "; font-style:normal");
+            fprintf(prstream, "; font-style:italic");
         }
+    } else {
+        fprintf(prstream, "; font-style:normal");
     }
 
-    if (strstr(fontalias, "Bold") != NULL) {
+    fontweight=get_fontweight(font);
+    if ((strstr(fontweight, "UltraLight") != NULL) ||
+        (strstr(fontweight, "ExtraLight") != NULL)) {
+        fprintf(prstream, "; font-weight:100");
+    } else if ((strstr(fontweight, "SemiLight") != NULL) ||
+               (strstr(fontweight, "Thin") != NULL)) {
+        fprintf(prstream, "; font-weight:200");
+    } else if (strstr(fontweight, "Light") != NULL) {
+        fprintf(prstream, "; font-weight:300");
+    } else if (strstr(fontweight, "Book") != NULL) {
+        fprintf(prstream, "; font-weight:500");
+    } else if (strstr(fontweight, "miBold") != NULL) {
+        fprintf(prstream, "; font-weight:600");
+    } else if ((strstr(fontweight, "ExtraBold") != NULL) ||
+               (strstr(fontweight, "Heavy") != NULL) ||
+               (strstr(fontweight, "UltraBold") != NULL)) {
+        fprintf(prstream, "; font-weight:800");
+    } else if (strstr(fontweight, "Bold") != NULL) {
         fprintf(prstream, "; font-weight:bold");
+    } else if ((strstr(fontweight, "ExtraBlack") != NULL) ||
+               (strstr(fontweight, "Ultra") != NULL)) {
+        fprintf(prstream, "; font-weight:900");
+    } else if (strstr(fontweight, "Black") != NULL) {
+        fprintf(prstream, "; font-weight:800");
     } else {
         fprintf(prstream, "; font-weight:normal");
     }
 
+    if ((strstr(fontfullname, "UltraCompressed") != NULL) ||
+        (strstr(fontfullname, "UltraCondensed") != NULL)) {
+        fprintf(prstream, "; font-stretch:ultra-condensed");
+    } else if ((strstr(fontfullname, "ExtraCompressed") != NULL) ||
+               (strstr(fontfullname, "ExtraCondensed") != NULL)) {
+        fprintf(prstream, "; font-stretch:extra-condensed");
+    } else if ((strstr(fontfullname, "SemiCondensed") != NULL) ||
+               (strstr(fontfullname, "Narrow") != NULL)) {
+        fprintf(prstream, "; font-stretch:semi-condensed");
+    } else if (strstr(fontfullname, "Condensed") != NULL) {
+        fprintf(prstream, "; font-stretch:condensed");
+    } else if ((strstr(fontfullname, "Wide") != NULL) ||
+               (strstr(fontfullname, "Poster") != NULL) ||
+               (strstr(fontfullname, "SemiExpanded") != NULL)) {
+        fprintf(prstream, "; font-stretch:semi-expanded");
+    } else if ((strstr(fontfullname, "ExtraExpanded") != NULL) ||
+               (strstr(fontfullname, "ExtraExtended") != NULL)) {
+        fprintf(prstream, "; font-stretch:extra-expanded");
+    } else if ((strstr(fontfullname, "UltraExpanded") != NULL) ||
+               (strstr(fontfullname, "UltraExtended") != NULL)) {
+        fprintf(prstream, "; font-stretch:ultra-expanded");
+    } else if ((strstr(fontfullname, "Expanded") != NULL) ||
+               (strstr(fontfullname, "Extended") != NULL)) {
+        fprintf(prstream, "; font-stretch:expanded");
+    }
+
     fprintf(prstream, "; font-size:%.4f", fsize);
 
     if (underline == TRUE) {
@@ -648,9 +741,9 @@ void svg_puttext(VPoint vp, char *s, int len, int font,
     }
 
     fprintf(prstream, "\" transform=\"matrix(%.4f,%.4f,%.4f,%.4f,%.4f,%.4f)\">",
-            tm->cxx, -tm->cyx,
-            -tm->cxy, tm->cyy,
-            convertX(vp.x), convertY(vp.y));
+            tm->cxx, tm->cyx,
+            -tm->cxy, -tm->cyy,
+            scaleval(vp.x), scaleval(vp.y));
 
     fprintf(prstream, escape_specials((unsigned char *) s, len));
 
@@ -665,5 +758,6 @@ void svg_leavegraphics(void)
         fprintf(prstream, "  </g>\n");
         data->group_is_open = FALSE;
     }
+    fprintf(prstream, " </g>\n");
     fprintf(prstream, "</svg>\n");
 }
