@@ -4,7 +4,7 @@
  * Home page: http://plasma-gate.weizmann.ac.il/Grace/
  * 
  * Copyright (c) 1991-1995 Paul J Turner, Portland, OR
- * Copyright (c) 1996-2002 Grace Development Team
+ * Copyright (c) 1996-2003 Grace Development Team
  * 
  * Maintained by Evgeny Stambulchik <fnevgeny@plasma-gate.weizmann.ac.il>
  * 
@@ -89,7 +89,7 @@ int init_t1(void)
         return (RETURN_FAILURE);
     }
     
-    nfonts = T1_Get_no_fonts();
+    nfonts = T1_GetNoFonts();
     if (nfonts < 1) {
         return (RETURN_FAILURE);
     }
